unit frmMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, nxLogging, Spin, ComCtrls;

const

  SC_NXLOGSERVERURL     = 'http://www.navimatix.de/loesungen/log-server/';

  SC_DEMOSERVERHOST     = 'nxgate02.navimatix.net';
  SC_DEMOSERVERPORT     = '8077';

type
  TMainForm = class(TForm)
    btnTrace: TButton;
    edtMessage: TLabeledEdit;
    edtModule: TLabeledEdit;
    btnDebug: TButton;
    btnInfo: TButton;
    btnWarn: TButton;
    btnError: TButton;
    btnException: TButton;
    btnFatal: TButton;
    edtTCPHost: TLabeledEdit;
    edtTCPPort: TLabeledEdit;
    cbUseTCPLog: TCheckBox;
    edtAppId: TLabeledEdit;
    Bevel1: TBevel;
    Bevel2: TBevel;
    btnLoop: TButton;
    Bevel3: TBevel;
    speLoopCount: TSpinEdit;
    pbAction: TProgressBar;
    lblBufferSize: TLabel;
    tmrTCPBufferView: TTimer;
    Label1: TLabel;
    btnAbout: TButton;
    lblInsertDemoServerdata: TLabel;
    lblServerQuestions: TLabel;
    edtMachineIdent: TLabeledEdit;
    procedure FormCreate(Sender: TObject);
    procedure btnTraceClick(Sender: TObject);
    procedure btnDebugClick(Sender: TObject);
    procedure btnInfoClick(Sender: TObject);
    procedure btnWarnClick(Sender: TObject);
    procedure btnErrorClick(Sender: TObject);
    procedure btnExceptionClick(Sender: TObject);
    procedure btnFatalClick(Sender: TObject);
    procedure btnLoopClick(Sender: TObject);
    procedure cbUseTCPLogClick(Sender: TObject);
    procedure tmrTCPBufferViewTimer(Sender: TObject);
    procedure btnAboutClick(Sender: TObject);
    procedure lblServerQuestionsClick(Sender: TObject);
    procedure lblInsertDemoServerdataClick(Sender: TObject);
  private
    fAppenderTCP      : TNxLogAppenderTCP;
  public
    { Public-Deklarationen }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

uses frmAbout, ShellAPI;

procedure StrResetLength(var S: WideString);
begin
  SetLength(S, StrLen(PWideChar(S)));
end;

function GetLocalComputerName: string;
var
  Count: DWORD;
  zws : widestring;
begin
  Count := MAX_COMPUTERNAME_LENGTH + 1;
  SetLength(zws, Count);
  if GetComputerName(PWideChar(zws), Count) then
    StrResetLength(zws)
  else
    zws := '';
  result := zws;
end;

procedure TMainForm.btnAboutClick(Sender: TObject);
begin
  AboutDialog.ShowModal;
end;

procedure TMainForm.btnDebugClick(Sender: TObject);
begin
  Logger.debug(edtModule.Text, edtMessage.Text);
end;

procedure TMainForm.btnErrorClick(Sender: TObject);
begin
  Logger.error(edtModule.Text, edtMessage.Text);
end;

procedure TMainForm.btnExceptionClick(Sender: TObject);
begin
  try
    raise Exception.Create(edtMessage.Text);
  except
    on e : exception do
    begin
      Logger.error(edtModule.Text, edtMessage.Text, NXLCAT_NONE, e);
    end;
  end;
end;

procedure TMainForm.btnFatalClick(Sender: TObject);
begin
  Logger.fatal(edtModule.Text, edtMessage.Text);
end;

procedure TMainForm.btnInfoClick(Sender: TObject);
begin
  Logger.info(edtModule.Text, edtMessage.Text);
end;

procedure TMainForm.btnTraceClick(Sender: TObject);
begin
  Logger.trace(edtModule.Text, edtMessage.Text);
end;

procedure TMainForm.btnWarnClick(Sender: TObject);
begin
  Logger.warn(edtModule.Text, edtMessage.Text);
end;

procedure TMainForm.cbUseTCPLogClick(Sender: TObject);
var cMachIdent  : String;
    i           : Integer;
    appender    : TNxLogAppender;
begin
  if cbUseTCPLog.Checked then
  begin
    cMachIdent  := 'local';
    Logger.initializeServerTCPLogging(edtAppId.Text, edtTCPHost.Text, StrToInt(edtTCPPort.Text), edtMachineIdent.Text, 10, '', '', [NXLL_TRACE, NXLL_DEBUG, NXLL_INFO, NXLL_WARN, NXLL_ERROR, NXLL_FATAL]);
    fAppenderTCP := nil;
    for i := 0 to Logger.AppenderCount - 1 do
    begin
      if Logger.Appenders[i] is TNxLogAppenderTCP then
      begin
        fAppenderTCP := Logger.Appenders[i] as TNxLogAppenderTCP;
        fAppenderTCP.MaxEntries := 5000000;
        break;
      end;
    end;

    edtAppId.Enabled  := false;
    edtAppId.Color    := clSilver;
    edtTCPHost.Enabled  := false;
    edtTCPHost.Color    := clSilver;
    edtTCPPort.Enabled  := false;
    edtTCPPort.Color    := clSilver;
    edtMachineIdent.Enabled := false;
    edtMachineIdent.Color := clSilver;

    tmrTCPBufferView.Enabled  := true;
  end else
  begin
    tmrTCPBufferView.Enabled  := false;

    edtAppId.Enabled  := true;
    edtAppId.Color    := clWindow;
    edtTCPHost.Enabled  := true;
    edtTCPHost.Color    := clWindow;
    edtTCPPort.Enabled  := true;
    edtTCPPort.Color    := clWindow;
    edtMachineIdent.Enabled := true;
    edtMachineIdent.Color := clWindow;

    for i := Logger.AppenderCount-1 downto 0 do
    begin
      if Logger.Appenders[0] is TNxLogAppenderTCP then
      begin
        appender  := Logger.deleteAppender(i);
        FreeAndNil(appender);
        fAppenderTCP  := nil;
      end;
    end;
  end;
end;

procedure TMainForm.btnLoopClick(Sender: TObject);
var i         : Integer;
begin
  pbAction.Min    := 0;
  pbAction.Max    := speLoopCount.Value;
  pbAction.Position := 0;
  for i := 0 to speLoopCount.Value - 1 do
  begin
    Logger.error(edtModule.Text, edtMessage.Text + ' ' + IntToStr(i));
    pbAction.Position := i;
    Application.ProcessMessages;
  end;
  pbAction.Position := pbAction.Max;
end;

procedure TMainForm.FormCreate(Sender: TObject);
var cAppId, cDir, cBase : String;
begin
  cAppId  := edtAppId.Text;
  cDir    := ExtractFilePath(ParamStr(0));
  cBase   := ExtractFileName(ParamStr(0)) + '.nxlog';
  try
    edtMachineIdent.Text  := GetLocalComputerName;
  except
    on e : exception do
    begin
      edtMachineIdent.Text  := 'demo_machine';
    end;
  end;
  Logger.initializeFileLogging(cAppId, cDir, cBase);
  Logger.CurrentLevel := NXLL_TRACE;
end;

procedure TMainForm.lblInsertDemoServerdataClick(Sender: TObject);
begin
  edtTCPHost.Text      := SC_DEMOSERVERHOST;
  edtTCPPort.Text      := SC_DEMOSERVERPORT;
  if cbUseTCPLog.Checked then
  begin
    cbUseTCPLog.Checked := false;
  end;
  cbUseTCPLog.Checked := true;
end;

procedure TMainForm.lblServerQuestionsClick(Sender: TObject);
begin
  shellexecute(self.Handle,pchar('open'),pchar(SC_NXLOGSERVERURL),'','',0);
end;

procedure TMainForm.tmrTCPBufferViewTimer(Sender: TObject);
begin
  if fAppenderTCP <> nil then
  begin
    lblBufferSize.Caption := IntToStr(fAppenderTCP.CurrentEntries);
  end;
end;

end.
