{ **************************************************************************** }
{ ***** Dateiname               : nxTailControl.pas ************************** }
{ ***** Autor                   : Navimatix GmbH, Matthias Heunecke ********** }
{ ***** Erstellt am             : 24.04.2014 ********************************* }
{ ***** Letzte nderung         : 24.04.2014 ********************************* }
{ ***** Zugehrigkeit           : nxLogging ********************************** }
{ ***** Beschreibung            : Enthlt eine Kontrollklasse fr das ******** }
{ ******************************* Starten und Stoppen von nxTail ************* }
{ **************************************************************************** }
{ ***** Gruppe                  : Freeware *********************************** }
{ ******************************* Freie Nutzung privat und kommerziell ******* }
{ **************************************************************************** }
{ ***** Version                 : 1.2.5.2 ************************************ }
{ **************************************************************************** }
{ ***** Status                  : aktiv ************************************** }
{ **************************************************************************** }
{ ***** URL                     : http://www.navimatix.de ******************** }
{ ***** URL                     : http://www.navimatix.de/loesungen/log-server }
{ ***** Email                   : info@navimatix.de ************************** }
{ ***** Adresse                 : Navimatix GmbH, Moritz-von-Rohr-Str. 1a, *** }
{ *****                           07745 Jena, Germany ************************ }
{ **************************************************************************** }
{ ***** Tutorial:  http://www.navimatix.de/loesungen/log-server/ ************* }
{ *****            logging-fuer-delphi-nxlogging/tutorial/ ******************* }
{ **************************************************************************** }

unit nxTailControl;

interface

uses SysUtils, Types, Classes;

const

  SCREG_APPLICATION             = 'Application';
  SCREG_TAILNAME                = 'nxTailName';

  HKLM_CURRENT_VERSION_WINDOWS  = 'Software\Microsoft\Windows';
  SCREG_CURRENTVERSION          = 'CurrentVersion';
  SCREG_PROGRAMFILESDIR         = 'ProgramFilesDir';

type

  ETailControl = class(Exception)

  end;

  TNxTailControl  = class(TComponent)
  protected
    fTailFilename       : String;
  protected
    function  getTailFilename : String; virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure startTail(aParam : String); virtual;
    procedure startTailAsAdmin(aParam : String); virtual;
  published

  end;


procedure Register;

implementation

uses ShellApi, Registry, Windows;

{ **************************************************************************** }
{ ***** Allgemeine Funktionen ************************************************ }
{ **************************************************************************** }

procedure Register;
begin
  RegisterComponents('Logging', [TNxTailControl]);
end;

function getProgramFilesFolder: string;
var r   : TRegIniFile;
    regfilepath : String;
begin
  result  := '';
  regfilepath := HKLM_CURRENT_VERSION_WINDOWS;
  r:=TRegIniFile.Create(KEY_READ);
  try
    r.RootKey:=HKEY_LOCAL_MACHINE;
    if r.OpenKey(regfilepath,false) then
    begin
      result  := r.ReadString(SCREG_CURRENTVERSION, SCREG_PROGRAMFILESDIR, '');
    end;
  finally
    FreeAndNil(r);
  end;
end;

function  getShellExecuteErrorText(aShellExecuteResult : Cardinal) : String;
begin
  case aShellExecuteResult of
    0 : result  := 'The operating system is out of memory or resources.';
    ERROR_FILE_NOT_FOUND : result  := 'The specified file was not found.';
    ERROR_PATH_NOT_FOUND : result  := 'The specified path was not found.';
    ERROR_BAD_FORMAT : result  := ' The .exe file is invalid (non-Win32 .exe or error in .exe image).';
    SE_ERR_ACCESSDENIED : result  := ' The operating system denied access to the specified file.';
    SE_ERR_ASSOCINCOMPLETE : result  := ' The file name association is incomplete or invalid.';
    SE_ERR_DDEBUSY : result  := ' The DDE transaction could not be completed because other DDE transactions were being processed.';
    SE_ERR_DDEFAIL : result  := ' The DDE transaction failed.';
    SE_ERR_DDETIMEOUT : result  := ' The DDE transaction could not be completed because the request timed out.';
    SE_ERR_DLLNOTFOUND : result  := ' The specified DLL was not found.';
    SE_ERR_NOASSOC : result  := ' There is no application associated with the given file name extension. This error will also be returned if you attempt to print a file that is not printable.';
    SE_ERR_OOM : result  := ' There was not enough memory to complete the operation.';
    SE_ERR_SHARE : result  := ' A sharing violation occurred.';
  else
    result  := '';
  end;
end;

procedure executeAsAdmin(aFilename, aParams, aDirectory : String; WaitForDone : Boolean = false);
var sei: TShellExecuteInfo;
begin
  ZeroMemory(@sei, SizeOf(sei));
  sei.cbSize := SizeOf(TShellExecuteInfo);
  sei.Wnd := 0;
  sei.fMask  := SEE_MASK_FLAG_NO_UI or SEE_MASK_NOCLOSEPROCESS;
  sei.lpVerb := PChar('runas');
  sei.lpFile := PChar(aFilename);
  if aParams <> '' then sei.lpParameters := PChar(aParams);
  sei.nShow := SW_SHOWNORMAL;
  if aDirectory <> '' then
  begin
    sei.lpDirectory := pchar(aDirectory);
  end else
  begin
    sei.lpDirectory := nil;
  end;
  if not ShellExecuteEx(@sei) then
  begin
    raise ETailControl.Create(Format('ShellExecuteEx() failed: %0:s', [getShellExecuteErrorText(sei.hProcess)]));
  end else
  begin
    if sei.hProcess <> 0 then
    begin
      if WaitForDone then WaitForSingleObject(sei.hProcess, INFINITE);
    end else
    begin
      raise ETailControl.Create(Format('ShellExecuteEx() failed: %0:s', [getShellExecuteErrorText(sei.hProcess)]));
    end;
    CloseHandle(sei.hProcess);
  end;
end;

procedure execute(aFilename, aParams, aDirectory : String; WaitForDone : Boolean = false);
var sei: TShellExecuteInfo;
begin
  ZeroMemory(@sei, SizeOf(sei));
  sei.cbSize := SizeOf(TShellExecuteInfo);
  sei.Wnd := 0;
  sei.fMask  := SEE_MASK_FLAG_NO_UI or SEE_MASK_NOCLOSEPROCESS;
  sei.lpFile := PChar(aFilename);
  if aParams <> '' then sei.lpParameters := PChar(aParams);
  sei.nShow := SW_SHOWNORMAL;
  if aDirectory <> '' then
  begin
    sei.lpDirectory := pchar(aDirectory);
  end else
  begin
    sei.lpDirectory := nil;
  end;
  if not ShellExecuteEx(@sei) then
  begin
    raise ETailControl.Create(Format('ShellExecuteEx() failed: %0:s', [getShellExecuteErrorText(sei.hProcess)]));
  end else
  begin
    if sei.hProcess <> 0 then
    begin
      if WaitForDone then WaitForSingleObject(sei.hProcess, INFINITE);
    end else
    begin
      raise ETailControl.Create(Format('ShellExecuteEx() failed: %0:s', [getShellExecuteErrorText(sei.hProcess)]));
    end;
    CloseHandle(sei.hProcess);
  end;
end;


{ **************************************************************************** }
{ ***** TNxTailControl ******************************************************* }
{ **************************************************************************** }

constructor TNxTailControl.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

end;

destructor TNxTailControl.Destroy;
begin

  inherited Destroy;
end;

function TNxTailControl.getTailFilename: String;
var r   : TRegIniFile;
    regfilepath : String;
    tailfile  : String;
begin
  // HKEY_LOCAL_MACHINE\SOFTWARE\NAVImatix\nxTail\Application\nxTailName
  regfilepath := 'Software\Navimatix\nxTail';
  r:=TRegIniFile.Create(KEY_READ);
  try
    r.RootKey:=HKEY_LOCAL_MACHINE;
    if r.OpenKey(regfilepath,false) then
    begin
      tailfile  := r.ReadString(SCREG_APPLICATION,SCREG_TAILNAME, '');
      if FileExists(tailfile) then
      begin
        result  := tailfile;
      end else
      begin
        raise ETailControl.Create(Format('file "%0:s" not exists', [tailfile]));
      end;
    end else
    begin
      tailfile  := ExtractFilePath(getProgramFilesFolder) + 'navimatix\nxTail\nxTail.exe';
      if FileExists(tailfile) then
      begin
        result  := tailfile;
      end else
      begin
        raise ETailControl.Create(Format('could not open registry key "%0:s" and nxTail.exe not at default position', ['HKEY_LOCAL_MACHINE\'+regfilepath]));
      end;
    end;
  finally
    FreeAndNil(r);
  end;
end;

procedure TNxTailControl.startTail(aParam: String);
var zs  : String;
begin
  zs  := getTailFilename;
  execute(zs, aParam, '', false);
end;

procedure TNxTailControl.startTailAsAdmin(aParam: String);
var zs  : String;
begin
  zs  := getTailFilename;
  executeAsAdmin(zs, aParam, '', false);
end;

end.
